#include <iostream>

using namespace std;

// Clase copiada del ejercicio 3. La manera en que habria que hacerlo es
// separando la declaracion de la implementacion de esta clase en ficheros
// .h y .cc separados, y aqui incluir el .h de la clase Complejo

class Complejo {

  friend ostream& operator<< (ostream& os, const Complejo& c);

public:

  Complejo (double r, double i) : real (r), imag (i) {}

  // Los argumentos de los operadores son constantes porque no
  // pueden modificarse dentro de los operadores.
  // Ademas, son todos metodos constantes (excepto el operador
  // de asignacion) porque tampoco modifican el argumento
  // implicito (this)
  Complejo operator+ (const Complejo& c) const;
  Complejo operator- (const Complejo& c) const;
  Complejo operator* (const Complejo& c) const;
  Complejo operator/ (const Complejo& c) const;
  Complejo operator* (double k) const;
  bool operator== (const Complejo& c) const;
  bool operator!= (const Complejo& c) const;
  Complejo& operator= (const Complejo& c);

private:
  double real;
  double imag;

};

Complejo Complejo::operator+ (const Complejo& c) const {

  Complejo res (0, 0);

  res.real = this->real + c.real;
  res.imag = this->imag + c.imag;

  return res;

}

Complejo Complejo::operator- (const Complejo& c) const {

  Complejo res (0, 0);
  
  res.real = this->real - c.real;
  res.imag = this->imag - c.imag;

  return res;

}

Complejo Complejo::operator* (const Complejo& c) const {

  Complejo res (0, 0);

  res.real = (this->real * c.real) - (this->imag * c.imag);
  res.imag = (this->real * c.imag) - (this->imag * c.real);

  return res;

}

Complejo Complejo::operator/ (const Complejo& c) const {

  Complejo res (0, 0);
  double denominador = (c.real * c.real) + (this->imag * this->imag);

  res.real = ((this->real * c.real) + (this->imag * c.imag)) / denominador;
  res.imag = (-(this->real * c.imag) + (this->imag * c.real)) / denominador;

  return res;

}

Complejo Complejo::operator* (double k) const {

  Complejo res (0, 0);

  res.real = this->real * k;
  res.imag = this->imag * k;

  return res;

}

bool Complejo::operator== (const Complejo& c) const {

  return (this->real == c.real) && (this->imag == c.imag);

}

bool Complejo::operator!= (const Complejo& c) const {

  return (this->real != c.real) || (this->imag != c.imag);

}

Complejo& Complejo::operator= (const Complejo& c) {

  this->real = c.real;
  this->imag = c.imag;

  return *this;

}

ostream& operator<< (ostream& os, const Complejo& c) {

  os << c.real << ((c.imag >= 0) ? "+" : "") << c.imag << "i";
  return os;

}


class Racional {

  friend ostream& operator<< (ostream& os, const Racional& r);
  friend istream& operator>> (istream& is, Racional& r);

public:

  Racional ();
  Racional (int n, int d);
  Racional (int a);
  Racional (double a);

  operator double ();
  operator Complejo ();

  int getNumerador () const;
  int getDenominador () const;

  Racional& operator= (const Racional& r);
  Racional& operator+= (const Racional& r);
  Racional& operator-= (const Racional& r);
  Racional& operator*= (const Racional& r);
  Racional& operator/= (const Racional& r);

  Racional operator+ (const Racional& r);
  Racional operator- (const Racional& r);
  Racional operator* (const Racional& r);
  Racional operator/ (const Racional& r);

  bool operator== (const Racional& r);
  bool operator!= (const Racional& r);
  bool operator> (const Racional& r);
  bool operator< (const Racional& r);
  bool operator>= (const Racional& r);
  bool operator<= (const Racional& r);

  Racional operator++ ();
  Racional operator++ (int x);

  Racional operator- () const;

  Racional& simplificar ();

private:

  int mcd (int a, int b) const;

  int num;
  int den;

};

Racional::Racional () : num(0), den(0) {
}

Racional::Racional (int n, int d) : num(n), den(d) {
}

Racional::Racional (int a) {
  num = a;
  den = 1;
}

Racional::Racional (double d) {
  num = d * 10000;
  den = 10000;

  this->simplificar();
}

int Racional::getNumerador () const {
  return num;
}

int Racional::getDenominador () const {
  return den;
}

Racional& Racional::operator= (const Racional& r) {
  this->num = r.num;
  this->den = r.den;

  return *this;
}

Racional& Racional::operator+= (const Racional& r) {
  this->num = (this->num * r.den) + (this->den * r.num);
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator-= (const Racional& r) {
  this->num = (this->num * r.den) - (this->den * r.num);
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator*= (const Racional& r) {
  this->num = this->num * r.num;
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator/= (const Racional& r) {
  this->num = this->num * r.den;
  this->den = this->den * r.num;

  this->simplificar ();

  return *this;
}

Racional Racional::operator+ (const Racional& r) {
  Racional res (0, 0);

  res.num = (this->num * r.den) + (this->den * r.num);
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator- (const Racional& r) {
  Racional res (0, 0);

  res.num = (this->num * r.den) - (this->den * r.num);
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator* (const Racional& r) {
  Racional res (0, 0);

  res.num = this->num * r.num;
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator/ (const Racional& r) {
  Racional res (0, 0);

  res.num = this->num * r.den;
  res.den = this->den * r.num;

  res.simplificar ();

  return res;
}

bool Racional::operator== (const Racional& r) {
  return (this->num * r.den) == (this->den * r.num);
}

bool Racional::operator!= (const Racional& r) {
  return (this->num * r.den) != (this->den * r.num);
}

bool Racional::operator> (const Racional& r) {
  return (this->num * r.den) > (this->den * r.num);
}

bool Racional::operator< (const Racional& r) {
  return (this->num * r.den) < (this->den * r.num);
}

bool Racional::operator>= (const Racional& r) {
  return (this->num * r.den) >= (this->den * r.num);
}

bool Racional::operator<= (const Racional& r) {
  return (this->num * r.den) <= (this->den * r.num);
}

Racional Racional::operator++ () {
  this->num += this->den;
  this->simplificar();

  return *this;
}

Racional Racional::operator++ (int x) {
  Racional temp = *this;

  this->num += this->den;
  this->simplificar();

  return temp;
}

Racional Racional::operator- () const {
  Racional temp = *this;
  temp.num *= -1;

  return temp;
}

int Racional::mcd (int a, int b) const {
  int mcd, temp, resto;
  mcd = abs (a);
  temp = b;

  while (temp > 0) {

    resto = mcd % temp;
    mcd = temp;
    temp = resto;

  }

  return mcd;
}

Racional& Racional::simplificar () {
  int div = mcd (this->num, this->den);

  this->num /= div;
  this->den /= div;

  return *this;
}

Racional::operator double () {
  return (double)num/(double)den;
}

Racional::operator Complejo () {
  Complejo res ((double)this->num / (double)this->den, 0);
  return res;
}

ostream& operator<< (ostream& os, const Racional& r) {
  os << r.num << "/" << r.den;
  return os;
}

istream& operator>> (istream& is, Racional& r) {
  is >> r.num;
  is.ignore (10, '/');
  is >> r.den;
  r.simplificar ();
  return is;
}


int main (void) {

  Racional r;

  cout << "Introduzca un racional: ";
  cin >> r;
  cout << endl;

  cout << r << " se convierte en el complejo " << static_cast<Complejo>(r) << endl;

  return 0;

}
