#include <iostream>

using namespace std;

class Complejo {

  friend ostream& operator<< (ostream& os, const Complejo& c);

public:

  Complejo (double r, double i) : real (r), imag (i) {}

  // Los argumentos de los operadores son constantes porque no
  // pueden modificarse dentro de los operadores.
  // Ademas, son todos metodos constantes (excepto el operador
  // de asignacion) porque tampoco modifican el argumento
  // implicito (this)
  Complejo operator+ (const Complejo& c) const;
  Complejo operator- (const Complejo& c) const;
  Complejo operator* (const Complejo& c) const;
  Complejo operator/ (const Complejo& c) const;
  Complejo operator* (double k) const;
  bool operator== (const Complejo& c) const;
  bool operator!= (const Complejo& c) const;
  Complejo& operator= (const Complejo& c);
  
private:
  double real;
  double imag;

};

Complejo Complejo::operator+ (const Complejo& c) const {

  Complejo res (0, 0);

  res.real = this->real + c.real;
  res.imag = this->imag + c.imag;

  return res;

}

Complejo Complejo::operator- (const Complejo& c) const {

  Complejo res (0, 0);
  
  res.real = this->real - c.real;
  res.imag = this->imag - c.imag;

  return res;

}

Complejo Complejo::operator* (const Complejo& c) const {

  Complejo res (0, 0);

  res.real = (this->real * c.real) - (this->imag * c.imag);
  res.imag = (this->real * c.imag) - (this->imag * c.real);

  return res;

}

Complejo Complejo::operator/ (const Complejo& c) const {

  Complejo res (0, 0);
  double denominador = (c.real * c.real) + (this->imag * this->imag);

  res.real = ((this->real * c.real) + (this->imag * c.imag)) / denominador;
  res.imag = (-(this->real * c.imag) + (this->imag * c.real)) / denominador;

  return res;

}

Complejo Complejo::operator* (double k) const {

  Complejo res (0, 0);

  res.real = this->real * k;
  res.imag = this->imag * k;

  return res;

}

bool Complejo::operator== (const Complejo& c) const {

  return (this->real == c.real) && (this->imag == c.imag);

}

bool Complejo::operator!= (const Complejo& c) const {

  return (this->real != c.real) || (this->imag != c.imag);

}

Complejo& Complejo::operator= (const Complejo& c) {

  this->real = c.real;
  this->imag = c.imag;

  return *this;

}

ostream& operator<< (ostream& os, const Complejo& c) {

  os << c.real << ((c.imag >= 0) ? "+" : "") << c.imag << "i";
  return os;

}

int main (void) {

  Complejo c1 (2, 3);
  Complejo c2 (7,-1);
  cout << c1 << endl << c2 << endl;

  Complejo c3 = c1 + c2;
  cout << c1 << " + " << c2 << " = " << c3 << endl;

  c3 = c2 - c1;
  cout << c2 << " - " << c1 << " = " << c3 << endl;

  c3 = c2 * c1;
  cout << c2 << " * " << c1 << " = " << c3 << endl;

  c3 = c2 / c1;
  cout << c2 << " / " << c1 << " = " << c3 << endl;

  if (c1 == c1)
    cout << c1 << " == " << c1 << " : true" << endl;

  if (c2 != c1)
    cout << c2 << " != " << c1 << " : true" << endl;

  return 0;

}
