#include <iostream>

using namespace std;

class Complejo {

  // Movemos todos los operadores fuera de la clase, anadimos el segundo argumento
  // explicito y hacemos que esas funciones sean amigas de la clase complejo
  friend ostream& operator<< (ostream& os, const Complejo& c);
  friend Complejo operator+ (const Complejo& c1, const Complejo& c2);
  friend Complejo operator- (const Complejo& c1, const Complejo& c2);
  friend Complejo operator* (const Complejo& c1, const Complejo& c2);
  friend Complejo operator/ (const Complejo& c1, const Complejo& c2);
  friend Complejo operator* (const Complejo& c, double k);
  friend bool operator== (const Complejo& c1, const Complejo& c2);
  friend bool operator!= (const Complejo& c1, const Complejo& c2);

public:

  Complejo (double r, double i) : real (r), imag (i) {}
  Complejo& operator= (const Complejo& c);
  
private:
  double real;
  double imag;

};

Complejo operator+ (const Complejo& c1, const Complejo& c2) {

  Complejo res (0, 0);

  res.real = c1.real + c2.real;
  res.imag = c1.imag + c2.imag;

  return res;

}

Complejo operator- (const Complejo& c1, const Complejo& c2) {

  Complejo res (0, 0);
  
  res.real = c1.real - c2.real;
  res.imag = c1.imag - c2.imag;

  return res;

}

Complejo operator* (const Complejo& c1, const Complejo& c2) {

  Complejo res (0, 0);

  res.real = (c1.real * c2.real) - (c1.imag * c2.imag);
  res.imag = (c1.real * c2.imag) - (c1.imag * c2.real);

  return res;

}

Complejo operator/ (const Complejo& c1, const Complejo& c2) {

  Complejo res (0, 0);
  double denominador = (c2.real * c2.real) + (c1.imag * c1.imag);

  res.real = ((c1.real * c2.real) + (c1.imag * c2.imag)) / denominador;
  res.imag = (-(c1.real * c2.imag) + (c1.imag * c2.real)) / denominador;

  return res;

}

Complejo operator* (const Complejo& c, double k) {

  Complejo res (0, 0);

  res.real = c.real * k;
  res.imag = c.imag * k;

  return res;

}

bool operator== (const Complejo& c1, const Complejo& c2) {

  return (c1.real == c2.real) && (c1.imag == c2.imag);

}

bool operator!= (const Complejo& c1, const Complejo& c2) {

  return (c1.real != c2.real) || (c1.imag != c2.imag);

}

Complejo& Complejo::operator= (const Complejo& c) {

  this->real = c.real;
  this->imag = c.imag;

  return *this;

}

ostream& operator<< (ostream& os, const Complejo& c) {

  os << c.real << ((c.imag > 0) ? "+" : "") << c.imag << "i";
  return os;

}

int main (void) {

  Complejo c1 (2, 3);
  Complejo c2 (7,-1);
  cout << c1 << endl << c2 << endl;

  Complejo c3 = c1 + c2;
  cout << c1 << " + " << c2 << " = " << c3 << endl;

  c3 = c2 - c1;
  cout << c2 << " - " << c1 << " = " << c3 << endl;

  c3 = c2 * c1;
  cout << c2 << " * " << c1 << " = " << c3 << endl;

  c3 = c2 / c1;
  cout << c2 << " / " << c1 << " = " << c3 << endl;

  if (c1 == c1)
    cout << c1 << " == " << c1 << " : true" << endl;

  if (c2 != c1)
    cout << c2 << " != " << c1 << " : true" << endl;

  return 0;

}
