#include <iostream>

using namespace std;

// Declaracion adelantada de la clase Matriz, para que la funcion amiga
// compile correctamente
class Matriz;

class Vector {

  friend Vector operator* (const Matriz& matriz, const Vector& vector);
  friend ostream& operator<< (ostream& os, const Vector& v);

public:

  Vector ();
  Vector (int tamano, double val);
  ~Vector ();
  Vector& operator= (const Vector& v);
  void leerDatos ();

private:
  int tamano;
  double* datos;
};

Vector::Vector () : tamano (0), datos (NULL) {}

Vector::Vector (int tam, double val = 0) : tamano (tam) {
  datos = new double[tamano];

  for (int i = 0; i < tamano; i++)
    datos[i] = val;
}

Vector::~Vector () {
  delete[] datos;
}

Vector& Vector::operator= (const Vector& v) {

  if (tamano == v.tamano) {
    for (int i = 0; i < tamano; i++)
      datos[i] = v.datos[i];
  }
  else {
    delete[] datos;

    tamano = v.tamano;
    datos = new double[v.tamano];

    for (int i = 0; i < tamano; i++)
      datos[i] = v.datos[i];
  }
  
  return *this;

}

void Vector::leerDatos () {
  cout << "Leemos el vector: " << endl;
  for (int i = 0; i < tamano; i++) {
    cout << "Pos " << i << ": ";
    cin >> datos[i];
  }
}

ostream& operator<< (ostream& os, const Vector& v) {
  for (int i = 0; i < v.tamano; i++)
    os << v.datos[i] << " ";
  os << endl;

  return os;
}


class Matriz {
  friend ostream& operator<< (ostream& os, const Matriz& m);
  friend Vector operator* (const Matriz& matriz, const Vector& vector);

public:

  Matriz (int r, int c, double val);
  ~Matriz ();
  void leerDatos();

private:

  int fils, cols;
  double** datos;

};

Matriz::Matriz (int filas, int columnas, double val = 0) : fils(filas), cols(columnas) {
  datos = new double*[fils];

  for (int i = 0; i < fils; i++)
    datos[i] = new double[cols];

  for (int i = 0; i < fils; i++)
    for (int j = 0; j < cols; j++)
      datos[i][j] = val;
}

Matriz::~Matriz () {
  for (int i = 0; i < fils; i++)
    delete[] datos[i];

  delete[] datos;
}

void Matriz::leerDatos () {
  cout << "Leemos la matriz: " << endl;
  for (int i = 0; i < fils; i++)
    for (int j = 0; j < cols; j++) {
      cout << "Pos [" << i << "][" << j << "]: ";
      cin >> datos[i][j];
    }
}

ostream& operator<< (ostream& os, const Matriz& m) {
  for (int i = 0; i < m.fils; i++) {
    for (int j = 0; j < m.cols; j++)
      os << m.datos[i][j] << " ";
    os << endl;
  }

  return os;
}

Vector operator* (const Matriz& matriz, const Vector& vector) {
  Vector res (matriz.fils);

  if (vector.tamano == matriz.cols) {
    for (unsigned i = 0; i < matriz.fils; i++) {
      int suma = 0;
      for (unsigned j = 0; j < matriz.cols; j++)
	suma += matriz.datos[i][j] * vector.datos[j];
      res.datos[i] = suma;
    }
  }
  else {
    cout << "La matriz y el vector proporcionados no se pueden multiplicar." << endl;
    exit(-1);
  }

  return res;
}

int main (void) {

  int tam, filas, columnas;

  cout << "Tamano del vector: ";
  cin >> tam;

  cout << "Filas de la matriz: ";
  cin >> filas;
  cout << "Columnas de la matriz: ";
  cin >> columnas;

  Vector v (tam);
  Matriz m (filas, columnas);

  v.leerDatos();
  m.leerDatos();

  cout << endl;
  cout << "El vector es: ";
  cout << v << endl;

  cout << endl;
  cout << "La matriz es: " << endl;
  cout << m << endl;

  Vector res = m * v;

  cout << endl;
  cout << "El vector resultado es: ";
  cout << res << endl;

  return 0;

}
