#include <iostream>

using namespace std;

class Racional {

  friend ostream& operator<< (ostream& os, const Racional& r);
  friend istream& operator>> (istream& is, Racional& r);

public:

  Racional ();
  Racional (int n, int d);

  int getNumerador () const;
  int getDenominador () const;

  Racional& operator= (const Racional& r);
  Racional& operator+= (const Racional& r);
  Racional& operator-= (const Racional& r);
  Racional& operator*= (const Racional& r);
  Racional& operator/= (const Racional& r);

  Racional operator+ (const Racional& r);
  Racional operator- (const Racional& r);
  Racional operator* (const Racional& r);
  Racional operator/ (const Racional& r);

  bool operator== (const Racional& r);
  bool operator!= (const Racional& r);
  bool operator> (const Racional& r);
  bool operator< (const Racional& r);
  bool operator>= (const Racional& r);
  bool operator<= (const Racional& r);

  Racional operator++ ();
  Racional operator++ (int x);

  Racional operator- () const;

  Racional& simplificar ();

private:

  int mcd (int a, int b) const;

  int num;
  int den;

};

Racional::Racional () : num(0), den(0) {
}

Racional::Racional (int n, int d) : num(n), den(d) {
}

int Racional::getNumerador () const {
  return num;
}

int Racional::getDenominador () const {
  return den;
}

Racional& Racional::operator= (const Racional& r) {
  this->num = r.num;
  this->den = r.den;

  return *this;
}

Racional& Racional::operator+= (const Racional& r) {
  this->num = (this->num * r.den) + (this->den * r.num);
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator-= (const Racional& r) {
  this->num = (this->num * r.den) - (this->den * r.num);
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator*= (const Racional& r) {
  this->num = this->num * r.num;
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator/= (const Racional& r) {
  this->num = this->num * r.den;
  this->den = this->den * r.num;

  this->simplificar ();

  return *this;
}

Racional Racional::operator+ (const Racional& r) {
  Racional res (0, 0);

  res.num = (this->num * r.den) + (this->den * r.num);
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator- (const Racional& r) {
  Racional res (0, 0);

  res.num = (this->num * r.den) - (this->den * r.num);
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator* (const Racional& r) {
  Racional res (0, 0);

  res.num = this->num * r.num;
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator/ (const Racional& r) {
  Racional res (0, 0);

  res.num = this->num * r.den;
  res.den = this->den * r.num;

  res.simplificar ();

  return res;
}

bool Racional::operator== (const Racional& r) {
  return (this->num * r.den) == (this->den * r.num);
}

bool Racional::operator!= (const Racional& r) {
  return (this->num * r.den) != (this->den * r.num);
}

bool Racional::operator> (const Racional& r) {
  return (this->num * r.den) > (this->den * r.num);
}

bool Racional::operator< (const Racional& r) {
  return (this->num * r.den) < (this->den * r.num);
}

bool Racional::operator>= (const Racional& r) {
  return (this->num * r.den) >= (this->den * r.num);
}

bool Racional::operator<= (const Racional& r) {
  return (this->num * r.den) <= (this->den * r.num);
}

Racional Racional::operator++ () {
  this->num += this->den;
  this->simplificar();

  return *this;
}

Racional Racional::operator++ (int x) {
  Racional temp = *this;

  this->num += this->den;
  this->simplificar();

  return temp;
}

Racional Racional::operator- () const {
  Racional temp = *this;
  temp.num *= -1;

  return temp;
}

int Racional::mcd (int a, int b) const {
  int mcd, temp, resto;
  mcd = abs (a);
  temp = b;

  while (temp > 0) {

    resto = mcd % temp;
    mcd = temp;
    temp = resto;

  }

  return mcd;
}

Racional& Racional::simplificar () {
  int div = mcd (this->num, this->den);

  this->num /= div;
  this->den /= div;

  return *this;
}

ostream& operator<< (ostream& os, const Racional& r) {
  os << r.num << "/" << r.den;
  return os;
}

istream& operator>> (istream& is, Racional& r) {
  is >> r.num;
  is.ignore (10, '/');
  is >> r.den;
  r.simplificar ();
  return is;
}

int main (void) {

  Racional r1, r2;

  cout << "Introduzca un racional: ";
  cin >> r1;
  cout << "Introduzca otro racional: ";
  cin >> r2;

  cout << "He leido: " << r1 << " y " << r2 << endl << endl;

  cout << r1 << " + " << r2 << " = " << (r1 + r2) << endl;
  cout << r1 << " - " << r2 << " = " << (r1 - r2) << endl;
  cout << r1 << " * " << r2 << " = " << (r1 * r2) << endl;
  cout << r1 << " / " << r2 << " = " << (r1 / r2) << endl;
  cout << endl;

  Racional r3;
  cout << "Introduzca un tercer racional: ";
  cin >> r3;

  cout << r3 << " += " << r1 << " => ";
  r3 += r1;
  cout << r3 << endl;

  cout << r3 << " -= " << r2 << " => ";
  r3 -= r2;
  cout << r3 << endl;

  cout << r3 << " *= " << r1 << " => ";
  r3 *= r1;
  cout << r3 << endl;

  cout << r3 << " /= " << r2 << " => ";
  r3 /= r2;
  cout << r3 << endl;
  cout << endl;

  if (r1 == r1)
    cout << r1 << " es igual a " << r1 << endl;

  if (r1 != r2)
    cout << r1 << " es distinto de " << r2 << endl;

  if (r1 > r2)
    cout << r1 << " es mayor que " << r2 << endl;

  if (r1 < r2)
    cout << r1 << " es menor que " << r2 << endl;

  if (r2 >= r1)
    cout << r2 << " es mayor o igual que " << r1 << endl;

  if (r2 <= r1)
    cout << r2 << " es menor o igual que " << r1 << endl;
  cout << endl;

  cout << "Valor original: " << r1 << endl;
  cout << "Valor pre-incrementado: " << ++r1 << endl;
  cout << "Valor posterior: " << r1 << endl;
  cout << endl;

  cout << "Valor original: " << r2 << endl;
  cout << "Valor post-incrementado: " << r2++ << endl;
  cout << "Valor posterior: " << r2 << endl;
  cout << endl;

  cout << "Invertimos: " << r1 << " : " << -r1 << endl;

  return 0;

}
