#include <iostream>

using namespace std;

class Racional {

  friend ostream& operator<< (ostream& os, const Racional& r);
  friend istream& operator>> (istream& is, Racional& r);

public:

  Racional ();
  Racional (int n, int d);
  Racional (int a);
  Racional (double a);

  int getNumerador () const;
  int getDenominador () const;

  Racional& operator= (const Racional& r);
  Racional& operator+= (const Racional& r);
  Racional& operator-= (const Racional& r);
  Racional& operator*= (const Racional& r);
  Racional& operator/= (const Racional& r);

  Racional operator+ (const Racional& r);
  Racional operator- (const Racional& r);
  Racional operator* (const Racional& r);
  Racional operator/ (const Racional& r);

  bool operator== (const Racional& r);
  bool operator!= (const Racional& r);
  bool operator> (const Racional& r);
  bool operator< (const Racional& r);
  bool operator>= (const Racional& r);
  bool operator<= (const Racional& r);

  Racional operator++ ();
  Racional operator++ (int x);

  Racional operator- () const;

  Racional& simplificar ();

private:

  int mcd (int a, int b) const;

  int num;
  int den;

};

Racional::Racional () : num(0), den(0) {
}

Racional::Racional (int n, int d) : num(n), den(d) {
}

Racional::Racional (int a) {
  num = a;
  den = 1;
}

Racional::Racional (double d) {
  num = d * 10000;
  den = 10000;

  this->simplificar();
}

int Racional::getNumerador () const {
  return num;
}

int Racional::getDenominador () const {
  return den;
}

Racional& Racional::operator= (const Racional& r) {
  this->num = r.num;
  this->den = r.den;

  return *this;
}

Racional& Racional::operator+= (const Racional& r) {
  this->num = (this->num * r.den) + (this->den * r.num);
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator-= (const Racional& r) {
  this->num = (this->num * r.den) - (this->den * r.num);
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator*= (const Racional& r) {
  this->num = this->num * r.num;
  this->den = this->den * r.den;

  this->simplificar ();

  return *this;
}

Racional& Racional::operator/= (const Racional& r) {
  this->num = this->num * r.den;
  this->den = this->den * r.num;

  this->simplificar ();

  return *this;
}

Racional Racional::operator+ (const Racional& r) {
  Racional res (0, 0);

  res.num = (this->num * r.den) + (this->den * r.num);
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator- (const Racional& r) {
  Racional res (0, 0);

  res.num = (this->num * r.den) - (this->den * r.num);
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator* (const Racional& r) {
  Racional res (0, 0);

  res.num = this->num * r.num;
  res.den = this->den * r.den;

  res.simplificar ();

  return res;
}

Racional Racional::operator/ (const Racional& r) {
  Racional res (0, 0);

  res.num = this->num * r.den;
  res.den = this->den * r.num;

  res.simplificar ();

  return res;
}

bool Racional::operator== (const Racional& r) {
  return (this->num * r.den) == (this->den * r.num);
}

bool Racional::operator!= (const Racional& r) {
  return (this->num * r.den) != (this->den * r.num);
}

bool Racional::operator> (const Racional& r) {
  return (this->num * r.den) > (this->den * r.num);
}

bool Racional::operator< (const Racional& r) {
  return (this->num * r.den) < (this->den * r.num);
}

bool Racional::operator>= (const Racional& r) {
  return (this->num * r.den) >= (this->den * r.num);
}

bool Racional::operator<= (const Racional& r) {
  return (this->num * r.den) <= (this->den * r.num);
}

Racional Racional::operator++ () {
  this->num += this->den;
  this->simplificar();

  return *this;
}

Racional Racional::operator++ (int x) {
  Racional temp = *this;

  this->num += this->den;
  this->simplificar();

  return temp;
}

Racional Racional::operator- () const {
  Racional temp = *this;
  temp.num *= -1;

  return temp;
}

int Racional::mcd (int a, int b) const {
  int mcd, temp, resto;
  mcd = abs (a);
  temp = b;

  while (temp > 0) {

    resto = mcd % temp;
    mcd = temp;
    temp = resto;

  }

  return mcd;
}

Racional& Racional::simplificar () {
  int div = mcd (this->num, this->den);

  this->num /= div;
  this->den /= div;

  return *this;
}

ostream& operator<< (ostream& os, const Racional& r) {
  os << r.num << "/" << r.den;
  return os;
}

istream& operator>> (istream& is, Racional& r) {
  is >> r.num;
  is.ignore (10, '/');
  is >> r.den;
  r.simplificar ();
  return is;
}

int main (void) {

  int a;
  double b;

  cout << "Introduzca un entero: ";
  cin >> a;
  cout << "Ahora introduzca un real: ";
  cin >> b;
  cout << endl;

  cout << a << " equivale al racional " << static_cast<Racional>(a) << endl;
  cout << b << " equivale al racional " << static_cast<Racional>(b) << endl;

  return 0;

}
